<?php
/**
 * @package         {package}
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2023 Logical-Arts-Extensions All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Plugin\Content\LartsMarkdown;
defined('_JEXEC') or die();

use Parsedown\Parsedown;

class Markdown
{
	static $plgCfg = null;				// plugin config

	public static function matchContent($text)
	{
		$result = [];
		
		$text = self::makePlainTextFromHTML($text);


		// Pattern for self-closing tags
		$pattern = '/\{\s*lartsmarkdown\s+([^}]+?\/)\s*\}/s';
		$matches = [];
		$count = @preg_match_all($pattern, $text, $matches);
		
		foreach ($matches[0] as $index => $match) {
			$result[] = [ 'match' => $matches[0][$index],
										'attribs' => $matches[1][$index],
										'content' => '',
									];

			// clean text from self closing patterns
			$text = str_replace($matches[0][$index], '', $text);
		}
		
		
		// Pattern for tags that enclose content
		$pattern = '/\{\s*lartsmarkdown\s+([^}]+?)\}(.*?)\{\s*\/lartsmarkdown\s*\}/s';
		$matches = [];
		$count = @preg_match_all($pattern, $text, $matches);

		// get matches...
		foreach ($matches[0] as $index => $match) {

			$result[] = [ 'match' => $matches[0][$index],
										'attribs' => $matches[1][$index],
										'content' => $matches[2][$index],
									];
		}


		foreach($result as $i => $entry) {
			$attribs = self::parseAttributes($entry['attribs']);
			$result[$i]['attribs'] = $attribs;
		}

		return $result;
	}
	
	public static function makePlainTextFromHTML($html)
	{
		// First, convert HTML line breaks to newline characters
		$withLineBreaks = str_replace(array('<br>', '<br/>', '<br />', '</p>'), "\n", $html);

		// Then, strip the remaining HTML tags
		$cleanText = strip_tags($withLineBreaks);

		// Display the cleaned text
		return $cleanText;	
	}
	
	public static function parseAttributes($text)
	{
		$attributes = [];
		$pattern = '/(\w+)\s*=\s*"([^"]*)"/';

		if (preg_match_all($pattern, $text, $attrMatches)) {
			foreach ($attrMatches[1] as $index => $key) {
				$attributes[$key] = $attrMatches[2][$index];
			}
		}

		return $attributes;
	}

	public static function parseFile($file)
	{
		$pd = new Parsedown;
		
		if(self::$plgCfg) {
			// enables automatic line breaks
			$pd->setBreaksEnabled(self::$plgCfg['enablebreaks']);
			// escapes markup (HTML)
			$pd->setMarkupEscaped(self::$plgCfg['markupescaped']);
			// prevents automatic linking of URLs
			$pd->setUrlsLinked(self::$plgCfg['urlslinked']);
			$pd->setSafeMode(self::$plgCfg['safemode']);
			$pd->setStrictMode(self::$plgCfg['strictmode']);
		}
		
		$html = $pd->parse($file);
		return $html;
	}
	
	public static function parseText($text)
	{
		$pd = new Parsedown;
		
		if(self::$plgCfg) {
			// enables automatic line breaks
			$pd->setBreaksEnabled(self::$plgCfg['enablebreaks']);
			// escapes markup (HTML)
			$pd->setMarkupEscaped(self::$plgCfg['markupescaped']);
			// prevents automatic linking of URLs
			$pd->setUrlsLinked(self::$plgCfg['urlslinked']);
			$pd->setSafeMode(self::$plgCfg['safemode']);
			$pd->setStrictMode(self::$plgCfg['strictmode']);
		}
		
		if(self::$plgCfg['enablebreaks']) {
			$text = str_replace('\n', "\n", $text);			
		}
		
		$html = $pd->text($text);
		return $html;
	}

	public static function parseLine($text)
	{
		$pd = new Parsedown;
		
		if(self::$plgCfg) {
			// enables automatic line breaks
			$pd->setBreaksEnabled(self::$plgCfg['enablebreaks']);
			// escapes markup (HTML)
			$pd->setMarkupEscaped(self::$plgCfg['markupescaped']);
			// prevents automatic linking of URLs
			$pd->setUrlsLinked(self::$plgCfg['urlslinked']);
			$pd->setSafeMode(self::$plgCfg['safemode']);
			$pd->setStrictMode(self::$plgCfg['strictmode']);
		}
		
		$html = $pd->line($text);
		return $html;
	}
	
}
