<?php
/**
 * @package         {package}
 * @version         24.3.1
 * 
 * @author          Logical-Arts <info@logical-arts.de>
 * @copyright       Copyright © 2023 Logical-Arts-Extensions All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 * @link            https://www.logical-arts-extensions.com
 */

namespace Larts\Plugin\Content\LartsMarkdown\Extension;

defined('_JEXEC') or die();
require_once( __DIR__. '/autoload.php');

use Joomla\CMS\Factory;
use Joomla\Event\DispatcherInterface;
use Joomla\Event\SubscriberInterface;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Event\Content\ContentPrepareEvent;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Form\FormHelper;

use Larts\Plugin\Content\LartsMarkdown\Markdown;
use Larts\Plugin\Content\LartsMarkdown\LartsPluginHelper;



final class LartsMarkdown extends CMSPlugin implements SubscriberInterface
{
	var $app = null;
	var $plgCfg = null;				// plugin config
  var $articleId = 0;
  var $itemId = 0;

	public function __construct(DispatcherInterface $dispatcher, array $config)
	{
		$this->app = Factory::getApplication();
		$this->setApplication($this->app);
		$this->autoloadLanguage = true;
		
		parent::__construct($dispatcher, $config);

 		$this->plgCfg = $this->params->toArray();

		// load language
		$lang = Factory::getLanguage();
		$lang->load('plg_content_lartsmarkdown', JPATH_PLUGINS.'/content/lartsmarkdown');

	}

  public static function getSubscribedEvents(): array
	{
		return [
			'onContentPrepare' => 'ContentPrepare',
		];
	}

  public function ContentPrepare($event)
  {
		if(!LartsPluginHelper::serverFetchDoc()) {
			return true;
		}
		
		if(is_a($event, 'Joomla\CMS\Event\Content\ContentPrepareEvent', false)) {
			// JVer 5
			$context = $event->getContext();
			$item = $event->getItem();
		}
		else if(is_a($event, 'Joomla\Event\Event', false)) {
			// JVer 4
			$args = $event->getArguments();
			$context = $args[0];
			$item = $args[1];
		}
		
    // Don't run this plugin when the content is being indexed
		if ($context == 'com_finder.indexer') {
			return true;
		}
    // Return if we are in backend
    if (!$this->app->isClient('site')) {
      return true;
    }
		// If the item does not have a text property there is nothing to do
		if (!property_exists($item, 'text')) {
			return;
		}

		if(empty($item->text)) {
			return;
		}

		$jInput = $this->app->getInput();
    
		$view	= $jInput->get('view');
		if ($view == 'tag')
      { return; }
    
		$this->articleId = $jInput->get('id');
    $this->itemId    = $jInput->getInt('Itemid');

    // pre check occurence of 'lartsmarkdown'...
		if(strpos($item->text, 'lartsmarkdown', 0) === false) {
			return;
		}

		// grab the tags
		$matches = Markdown::matchContent($item->text);
		
 		foreach ($matches as $match) {
			
			$cfg = new \stdClass();
			$cfg = (object)$this->plgCfg;
			

			$html = "";

			if(isset($cfg->file)) {
				$html .= $this->renderFile($cfg->file, $cfg);
			}
			else {	// text inside tags
				$html .= $this->renderText($match['content'], $cfg);
			}


			$item->text = str_replace($match['match'], $html, $item->text);
		}

		return false;
	}

  public function renderLine($line, $cfg)
  {
		Markdown::$plgCfg = (array)$cfg;
		$html = Markdown::parseLine($line);
		return $html;
	}
	
  public function renderText($text, $cfg)
  {

		if(isset($cfg->plain) && $cfg->plain) {
			$html = $text;
		}
		else {
			Markdown::$plgCfg = (array)$cfg;
			$html = Markdown::parseText($text);
		}

		if($cfg->enablebreaks) {
			$html = str_replace("\n", '<br>', $html);			
		}
		
		return $html;
	}

	public function renderFile($file, $cfg)
  {
		// strip marks
		$file = str_replace('"', '', $file);    // unquote
		$filepath = JPATH_ROOT. '/'.$file;

		if(!file_exists($filepath)) {
			return '<p>File not found: "'.$file.'"</p>';
		}

		if(!($text = file_get_contents($filepath))) {
			return '<p>ERROR loading file</p>';
		}

		$pinfo = pathinfo($filepath);

		if($pinfo['extension'] == 'md')
		{
			$cfg->monospaced = false;
			Markdown::$plgCfg = (array)$cfg;
			return Markdown::parseFile($text);
		}
		else {
			return '<p>Invalid extension "' .$pinfo['extension']. '"</p>';
		}

		return '';
	}

}


